#!/usr/bin/env python3
"""
Report Card Data Aggregation Module for Dane County School Dashboard

This module takes raw school virus data from retrieve_virus_data.py and creates
report card data with normalized assay names, academic year assignments, and
aggregated results based on assay groups.

The module handles:
- Normalized assay_target names from assay_config.toml
- Academic year assignment (Aug 15 - Jun 1 format)
- Aggregated results for pathogen groups (e.g. SARS-CoV-2 from N1+N2)
- Assay system precedence rules
- Excel output generation

Usage:
    from create_report_card_data import create_report_card_data
    
    # Get raw school virus data
    raw_df = retrieve_school_virus_data()
    
    # Create report card data
    report_card_df = create_report_card_data(raw_df)
"""

import sys
import os
import tomllib
from pathlib import Path
from typing import Dict, List, Any, Optional, Tuple
import pandas as pd
import numpy as np
from rich.console import Console
from rich.progress import Progress, SpinnerColumn, TextColumn, BarColumn
from rich.table import Table
import pickle
import gzip
from datetime import datetime, date
import re

# Load environment variables from .env file if running locally
try:
    from dotenv import load_dotenv
    # Look for .env file in the project root
    env_path = Path(__file__).parent.parent.parent / '.env'
    if env_path.exists():
        load_dotenv(env_path)
except ImportError:
    # dotenv not available, rely on environment variables being set externally
    pass

# Import the school virus data retrieval module
from retrieve_virus_data import retrieve_school_virus_data, save_to_cache, load_from_cache


def load_sites_config(sites_path: Optional[Path] = None) -> Dict[str, Any]:
    """
    Load sites configuration from sites.toml file.
    
    Args:
        sites_path: Path to sites.toml file. If None, looks in current directory.
        
    Returns:
        Dictionary with sites configuration
    """
    if sites_path is None:
        sites_path = Path(__file__).parent / "sites.toml"
    else:
        sites_path = Path(sites_path)
    
    if not sites_path.exists():
        raise FileNotFoundError(f"Sites configuration file not found: {sites_path}")
    
    try:
        with open(sites_path, 'rb') as f:
            config = tomllib.load(f)
        return config
    except Exception as e:
        raise Exception(f"Failed to load sites configuration from {sites_path}: {e}")


def load_assay_config(config_path: Optional[Path] = None) -> Dict[str, Any]:
    """
    Load assay configuration from assay_config.toml file.
    
    Args:
        config_path: Path to assay_config.toml file. If None, looks in current directory.
        
    Returns:
        Dictionary with assay configuration
    """
    if config_path is None:
        config_path = Path(__file__).parent / "assay_config.toml"
    else:
        config_path = Path(config_path)
    
    if not config_path.exists():
        raise FileNotFoundError(f"Assay configuration file not found: {config_path}")
    
    try:
        with open(config_path, 'rb') as f:
            config = tomllib.load(f)
        return config
    except Exception as e:
        raise Exception(f"Failed to load assay configuration from {config_path}: {e}")


def add_academic_year(df: pd.DataFrame, date_column: str = 'start_time') -> pd.DataFrame:
    """
    Add academic year column based on date (Aug 15 - Jun 1 format).
    
    Args:
        df: DataFrame with date column
        date_column: Name of the date column
        
    Returns:
        DataFrame with academic_year column added
    """
    if df.empty or date_column not in df.columns:
        return df
    
    result_df = df.copy()
    
    # Ensure date column is datetime
    if not pd.api.types.is_datetime64_any_dtype(result_df[date_column]):
        result_df[date_column] = pd.to_datetime(result_df[date_column], errors='coerce')
    
    def calculate_academic_year(date_val):
        if pd.isna(date_val):
            return None
        
        # Academic year starts August 15 and ends June 1
        if date_val.month >= 8 or (date_val.month == 8 and date_val.day >= 15):
            # Fall semester - academic year is current year to next year
            start_year = date_val.year
            end_year = date_val.year + 1
        elif date_val.month <= 6 or (date_val.month == 6 and date_val.day <= 1):
            # Spring semester - academic year is previous year to current year
            start_year = date_val.year - 1
            end_year = date_val.year
        else:
            # Summer (June 2 - August 14) - ambiguous, assign to previous academic year
            start_year = date_val.year - 1
            end_year = date_val.year
        
        return f"{start_year}-{str(end_year)[2:]}"
    
    result_df['academic_year'] = result_df[date_column].apply(calculate_academic_year)
    
    return result_df


def add_school_type(df: pd.DataFrame, sites_config: Dict[str, Any]) -> pd.DataFrame:
    """
    Add school type column based on sites.toml configuration.
    
    Args:
        df: DataFrame with facility_name column
        sites_config: Sites configuration dictionary
        
    Returns:
        DataFrame with school_type column added
    """
    if df.empty or 'facility_name' not in df.columns:
        return df
    
    result_df = df.copy()
    
    # Create mapping from school name to type
    school_type_mapping = {}
    schools = sites_config.get('schools', {})
    
    for school_key, school_info in schools.items():
        school_name = school_info.get('name', school_key)
        school_type = school_info.get('type', 'unknown')
        school_type_mapping[school_name] = school_type
    
    # Apply school type mapping
    result_df['school_type'] = result_df['facility_name'].map(school_type_mapping)
    
    # For unmapped schools, set as unknown
    result_df['school_type'] = result_df['school_type'].fillna('unknown')
    
    return result_df


def normalize_assay_target_names(df: pd.DataFrame, config: Dict[str, Any]) -> pd.DataFrame:
    """
    Add normalized assay_target names using virus_normalization mapping from config.
    
    Args:
        df: DataFrame with assay_target column
        config: Assay configuration dictionary
        
    Returns:
        DataFrame with normalized_assay_target column added
    """
    if df.empty or 'assay_target' not in df.columns:
        return df
    
    result_df = df.copy()
    
    # Get virus normalization mapping
    virus_mapping = config.get('virus_normalization', {}).get('mapping', {})
    
    # Apply normalization mapping
    result_df['normalized_assay_target'] = result_df['assay_target'].map(virus_mapping)
    
    # For unmapped targets, use original name
    result_df['normalized_assay_target'] = result_df['normalized_assay_target'].fillna(result_df['assay_target'])
    
    return result_df


def transform_sars_cov2_results(df: pd.DataFrame) -> pd.DataFrame:
    """
    Transform SARS-CoV-2 results by combining N1 and N2 targets.
    If either N1 OR N2 is Positive, the combined result is Positive.
    
    Args:
        df: DataFrame with assay results
        
    Returns:
        DataFrame with SARS-CoV-2 combined results
    """
    if df.empty:
        return df
    
    result_df = df.copy()
    sars_targets = ['SARS CoV2 N1', 'SARS CoV2 N2']
    sars_data = result_df[result_df['assay_target'].isin(sars_targets)].copy()
    
    if sars_data.empty:
        return result_df
    
    # Group by cartridge_id to combine N1 and N2 results
    if 'cartridge_id' not in sars_data.columns:
        return result_df
    
    sars_combined_rows = []
    
    for cartridge_id, group in sars_data.groupby('cartridge_id'):
        if len(group) < 2:  # Need both N1 and N2
            continue
            
        # Check if any N1 or N2 is positive
        positive_results = group[group['qualitative_result'].isin(['Positive', 'positive', 'POSITIVE'])]
        
        if len(positive_results) > 0:
            # Create combined positive result
            combined_record = group.iloc[0].copy()  # Use first record as template
            combined_record['assay_target'] = 'SARS-CoV-2'
            combined_record['normalized_assay_target'] = 'SARS-CoV-2'
            combined_record['qualitative_result'] = 'Positive'
            combined_record['is_combined_result'] = True
            
            # Sum quantitative values if available
            if 'average_result' in group.columns:
                combined_record['average_result'] = group['average_result'].sum()
            if 'copies_per_reaction' in group.columns:
                combined_record['copies_per_reaction'] = group['copies_per_reaction'].sum()
                
            sars_combined_rows.append(combined_record)
        else:
            # All negative - create combined negative result
            combined_record = group.iloc[0].copy()
            combined_record['assay_target'] = 'SARS-CoV-2'
            combined_record['normalized_assay_target'] = 'SARS-CoV-2'
            combined_record['qualitative_result'] = 'Negative'
            combined_record['is_combined_result'] = True
            sars_combined_rows.append(combined_record)
    
    # Add combined results and remove original N1/N2 records
    if sars_combined_rows:
        # Remove original SARS-CoV-2 N1 and N2 records
        result_df = result_df[~result_df['assay_target'].isin(sars_targets)].copy()
        
        # Mark remaining records as not combined
        result_df['is_combined_result'] = False
        
        # Add combined results
        combined_df = pd.DataFrame(sars_combined_rows)
        result_df = pd.concat([result_df, combined_df], ignore_index=True)
    
    return result_df


def create_date_ranges(df: pd.DataFrame) -> pd.DataFrame:
    """
    Create logical date range groupings for samples with similar start_time and end_time.
    
    Args:
        df: DataFrame with start_time and end_time columns
        
    Returns:
        DataFrame with date_range column added
    """
    if df.empty or 'start_time' not in df.columns:
        return df
    
    result_df = df.copy()
    
    # Convert to datetime if needed
    if not pd.api.types.is_datetime64_any_dtype(result_df['start_time']):
        result_df['start_time'] = pd.to_datetime(result_df['start_time'], errors='coerce')
    
    if 'end_time' in result_df.columns and not pd.api.types.is_datetime64_any_dtype(result_df['end_time']):
        result_df['end_time'] = pd.to_datetime(result_df['end_time'], errors='coerce')
    
    # Create date range labels based on start_time (group by week)
    result_df['sample_week'] = result_df['start_time'].dt.to_period('W')
    result_df['date_range'] = result_df['sample_week'].astype(str)
    
    return result_df



def aggregate_assay_results(df: pd.DataFrame, assay_groups: Dict[str, List[str]]) -> pd.DataFrame:
    """
    Create aggregated results for assay groups.
    
    For each group, if any of the constituent assays are positive, the group is positive.
    
    Args:
        df: DataFrame with individual assay results
        assay_groups: Dictionary mapping group names to assay target lists
        
    Returns:
        DataFrame with aggregated results added
    """
    if df.empty:
        return df
    
    result_df = df.copy()
    aggregated_rows = []
    
    # Group by facility, date, and assay_system for aggregation
    grouping_cols = ['facility_name', 'start_time', 'assay_system']
    available_grouping_cols = [col for col in grouping_cols if col in result_df.columns]
    
    if not available_grouping_cols:
        return result_df
    
    # Add academic_year to grouping if available
    if 'academic_year' in result_df.columns:
        available_grouping_cols.append('academic_year')
    
    for group_name, target_list in assay_groups.items():
        # Find samples that have any of the targets in this group
        group_df = result_df[result_df['assay_target'].isin(target_list)].copy()
        
        if group_df.empty:
            continue
        
        # Group by facility/date/assay_system and aggregate
        for group_key, group_data in group_df.groupby(available_grouping_cols):
            # Check if any assay in this group is positive
            positive_assays = group_data[group_data['qualitative_result'].isin(['positive', 'Positive', 'POSITIVE'])]
            
            if len(positive_assays) > 0:
                # Create aggregated record
                agg_record = group_data.iloc[0].copy()  # Use first record as template
                agg_record['assay_target'] = group_name
                agg_record['normalized_assay_target'] = group_name
                agg_record['qualitative_result'] = 'positive'
                agg_record['is_aggregated'] = True
                
                # Sum quantitative values if available
                if 'average_result' in group_data.columns:
                    agg_record['average_result'] = group_data['average_result'].sum()
                if 'copies_per_reaction' in group_data.columns:
                    agg_record['copies_per_reaction'] = group_data['copies_per_reaction'].sum()
                
                aggregated_rows.append(agg_record)
    
    # Add aggregated rows to result
    if aggregated_rows:
        result_df['is_aggregated'] = False  # Mark original records
        aggregated_df = pd.DataFrame(aggregated_rows)
        result_df = pd.concat([result_df, aggregated_df], ignore_index=True)
    
    return result_df


def apply_assay_precedence(df: pd.DataFrame, config: Dict[str, Any]) -> pd.DataFrame:
    """
    Apply assay precedence rules when multiple assay systems test the same pathogen.
    
    Args:
        df: DataFrame with assay results
        config: Assay configuration dictionary
        
    Returns:
        DataFrame with precedence rules applied
    """
    if df.empty:
        return df
    
    precedence_order = config.get('precedence', {}).get('order', [])
    if not precedence_order:
        return df
    
    result_df = df.copy()
    
    # Add priority based on precedence order
    priority_map = {assay: i for i, assay in enumerate(precedence_order)}
    result_df['assay_priority'] = result_df['assay_system'].map(priority_map)
    result_df['assay_priority'] = result_df['assay_priority'].fillna(999)  # Low priority for unmapped
    
    # Group by facility, date, and normalized_assay_target
    grouping_cols = ['facility_name', 'start_time', 'normalized_assay_target']
    available_grouping_cols = [col for col in grouping_cols if col in result_df.columns]
    
    if 'academic_year' in result_df.columns:
        available_grouping_cols.append('academic_year')
    
    # For each group, keep only the highest priority assay
    def select_highest_priority(group):
        if len(group) == 1:
            return group
        
        # Sort by priority (lower number = higher priority)
        group_sorted = group.sort_values('assay_priority')
        return group_sorted.iloc[[0]]  # Return first (highest priority) as DataFrame
    
    result_df = result_df.groupby(available_grouping_cols, group_keys=False).apply(select_highest_priority).reset_index(drop=True)
    
    # Remove temporary priority column
    result_df = result_df.drop('assay_priority', axis=1, errors='ignore')
    
    return result_df


def create_report_card_data(raw_df: pd.DataFrame, 
                           test_school: str = None,
                           show_progress: bool = True,
                           cache_file: Optional[Path] = None,
                           use_cache: bool = False) -> pd.DataFrame:
    """
    Create report card data from raw school virus data.
    
    This function processes raw school surveillance data and creates a report card
    with normalized assay names, academic year assignments, and aggregated results.
    
    Args:
        raw_df: Raw school virus data from retrieve_school_virus_data
        test_school: If provided, filter to this school for testing (e.g., "Oregon High School")
        show_progress: Whether to show progress indicators
        cache_file: Path to cache file for saving/loading processed data
        use_cache: If True and cache exists, load from cache instead of processing
        
    Returns:
        pandas DataFrame with report card structure:
        - facility_name: School name
        - start_time: Sample date
        - academic_year: Academic year (YYYY-YY format)
        - assay_system: Assay system used
        - assay_target: Original assay target name
        - normalized_assay_target: Normalized assay target name
        - qualitative_result: Test result (positive/negative/etc)
        - average_result: Quantitative result if available
        - is_aggregated: Whether this is an aggregated result
    """
    console = Console()
    
    # Try to load from cache if requested
    if use_cache and cache_file and cache_file.exists():
        if show_progress:
            console.print(f"[cyan]Loading report card data from cache: {cache_file}[/cyan]")
        
        try:
            df, cache_info = load_from_cache(cache_file)
            age_days = (datetime.now() - cache_info['timestamp']).days
            
            if show_progress:
                console.print(f"[green]✓ Loaded {len(df)} report card records from cache[/green]")
                console.print(f"  Cache age: {age_days} days old")
                console.print(f"  Created: {cache_info['timestamp'].strftime('%Y-%m-%d %H:%M:%S')}")
                
                if age_days > 7:
                    console.print(f"[yellow]⚠ Cache is {age_days} days old. Consider refreshing.[/yellow]")
            
            return df
            
        except Exception as e:
            console.print(f"[yellow]Failed to load cache: {e}[/yellow]")
            console.print("[cyan]Falling back to fresh data processing...[/cyan]")
    
    if raw_df.empty:
        console.print("[yellow]Warning: Empty input DataFrame[/yellow]")
        return pd.DataFrame()
    
    if show_progress:
        console.print("[blue]Creating report card data from raw school virus data...[/blue]")
    
    try:
        with Progress(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            BarColumn(),
            TextColumn("[progress.percentage]{task.percentage:>3.0f}%"),
            console=console,
            disable=not show_progress
        ) as progress:
            
            main_task = progress.add_task("Processing school virus data...", total=100)
            
            # Step 1: Load configurations
            progress.update(main_task, description="Loading configurations...", completed=5)
            config = load_assay_config()
            sites_config = load_sites_config()
            
            # Step 2: Basic data cleaning and filtering
            progress.update(main_task, description="Cleaning and filtering data...", completed=10)
            df = raw_df.copy()
            
            # Filter to test school if specified
            if test_school:
                df = df[df['facility_name'] == test_school].copy()
                if df.empty:
                    console.print(f"[yellow]Warning: No data found for school: {test_school}[/yellow]")
                    return pd.DataFrame()
                
                if show_progress:
                    console.print(f"[cyan]Filtered to {test_school}: {len(df)} records[/cyan]")
            
            # Remove records with missing essential data
            df = df.dropna(subset=['facility_name', 'start_time', 'assay_target', 'assay_system'])
            
            # Step 3: Add academic year
            progress.update(main_task, description="Adding academic year...", completed=20)
            df = add_academic_year(df, 'start_time')
            
            # Step 4: Transform SARS-CoV-2 results (combine N1 and N2)
            progress.update(main_task, description="Transforming SARS-CoV-2 results...", completed=25)
            df = transform_sars_cov2_results(df)
            
            # Step 5: Add normalized assay target names
            progress.update(main_task, description="Normalizing assay target names...", completed=35)
            df = normalize_assay_target_names(df, config)
            
            # Step 6: Add school type from sites config
            progress.update(main_task, description="Adding school types...", completed=45)
            df = add_school_type(df, sites_config)
            
            # Step 7: Create date ranges
            progress.update(main_task, description="Creating date ranges...", completed=55)
            df = create_date_ranges(df)
            
            # Step 8: Apply assay precedence rules (simplified for BioMarkX only)
            progress.update(main_task, description="Applying assay precedence rules...", completed=70)
            df = apply_assay_precedence(df, config)
            
            # Step 9: Final sorting and cleanup
            progress.update(main_task, description="Final sorting and cleanup...", completed=85)
            
            # Sort by facility, date, and assay target
            sort_columns = ['facility_name', 'start_time', 'normalized_assay_target']
            df = df.sort_values(sort_columns).reset_index(drop=True)
            
            # Step 10: Final processing
            progress.update(main_task, description="Finalizing data processing...", completed=95)
            
            progress.update(main_task, description="Report card data creation complete", completed=100)
        
        if show_progress:
            console.print(f"[green]✓ Successfully created report card data: {len(df)} records[/green]")
            
            # Show summary statistics
            if not df.empty:
                unique_schools = df['facility_name'].nunique()
                unique_targets = df['normalized_assay_target'].nunique()
                academic_years = sorted(df['academic_year'].dropna().unique())
                
                console.print(f"[cyan]Summary: {unique_schools} schools, {unique_targets} targets, academic years: {academic_years}[/cyan]")
        
        # Save to cache if cache_file is specified
        if cache_file and not df.empty:
            if show_progress:
                console.print(f"[cyan]Saving report card data to cache: {cache_file}[/cyan]")
            
            metadata = {
                'test_school': test_school,
                'row_count': len(df),
                'unique_schools': df['facility_name'].nunique() if not df.empty else 0,
                'unique_targets': df['normalized_assay_target'].nunique() if not df.empty else 0,
                'academic_years': sorted(df['academic_year'].dropna().unique().tolist()) if not df.empty else [],
                'columns': list(df.columns)
            }
            
            try:
                save_to_cache(df, cache_file, metadata)
                if show_progress:
                    console.print(f"[green]✓ Saved {len(df)} records to cache[/green]")
            except Exception as e:
                console.print(f"[yellow]Failed to save cache: {e}[/yellow]")
        
        return df
        
    except Exception as e:
        error_msg = f"Failed to create report card data: {e}"
        if show_progress:
            console.print(f"[red]✗ {error_msg}[/red]")
        raise Exception(error_msg)


def save_to_excel(df: pd.DataFrame, output_path: Path, include_viz_data: bool = True) -> None:
    """
    Save report card data to Excel file.
    
    Args:
        df: Report card DataFrame
        output_path: Path for Excel output file
    """
    if df.empty:
        raise ValueError("Cannot save empty DataFrame to Excel")
    
    # Ensure output directory exists
    output_path.parent.mkdir(parents=True, exist_ok=True)
    
    # Save to Excel with formatting
    with pd.ExcelWriter(output_path, engine='openpyxl') as writer:
        # Main data sheet
        df.to_excel(writer, sheet_name='Report Card Data', index=False)
        
        # Summary sheet
        summary_data = {
            'Metric': [
                'Total Records',
                'Unique Schools', 
                'Unique Targets',
                'Academic Years',
                'Date Range',
                'Generated On'
            ],
            'Value': [
                len(df),
                df['facility_name'].nunique(),
                df['normalized_assay_target'].nunique(),
                ', '.join(sorted(df['academic_year'].dropna().unique())),
                f"{df['start_time'].min().date()} to {df['start_time'].max().date()}" if not df['start_time'].isna().all() else 'N/A',
                datetime.now().strftime('%Y-%m-%d %H:%M:%S')
            ]
        }
        
        summary_df = pd.DataFrame(summary_data)
        summary_df.to_excel(writer, sheet_name='Summary', index=False)
        
        # School breakdown sheet
        school_summary = df.groupby('facility_name').agg({
            'start_time': 'count',
            'normalized_assay_target': 'nunique',
            'academic_year': lambda x: ', '.join(sorted(x.dropna().unique())),
            'school_type': 'first'  # School type should be consistent per school
        }).reset_index()
        school_summary.columns = ['School', 'Total Samples', 'Unique Targets', 'Academic Years', 'School Type']
        school_summary.to_excel(writer, sheet_name='School Summary', index=False)
        
        # Note: Visualization data functionality removed - use create_visualizations.py for plot data
        
        # Virus summary sheet
        virus_summary = df.groupby('normalized_assay_target').agg({
            'start_time': 'count',
            'facility_name': 'nunique',
            'qualitative_result': lambda x: (x.isin(['Positive', 'positive', 'POSITIVE']).sum(), len(x))
        }).reset_index()
        
        # Expand the qualitative_result tuple
        virus_summary[['Positive_Count', 'Total_Count']] = pd.DataFrame(virus_summary['qualitative_result'].tolist(), index=virus_summary.index)
        virus_summary['Positive_Rate'] = virus_summary['Positive_Count'] / virus_summary['Total_Count']
        virus_summary = virus_summary.drop('qualitative_result', axis=1)
        virus_summary.columns = ['Virus Target', 'Total Samples', 'Unique Schools', 'Positive Count', 'Total Count', 'Positive Rate']
        virus_summary.to_excel(writer, sheet_name='Virus Summary', index=False)


def get_report_card_summary(df: pd.DataFrame) -> Dict[str, Any]:
    """
    Get summary statistics for report card data.
    
    Args:
        df: Report card DataFrame
        
    Returns:
        Dictionary with summary statistics
    """
    if df.empty:
        return {"total_records": 0, "message": "No data available"}
    
    summary = {
        "total_records": len(df),
        "unique_schools": df['facility_name'].nunique(),
        "unique_targets": df['normalized_assay_target'].nunique(),
        "academic_years": sorted(df['academic_year'].dropna().unique().tolist()),
        "date_range": {
            "earliest": df['start_time'].min(),
            "latest": df['start_time'].max()
        },
        "schools": sorted(df['facility_name'].unique().tolist()),
        "targets": sorted(df['normalized_assay_target'].unique().tolist()),
        "assay_systems": sorted(df['assay_system'].unique().tolist()),
        "aggregated_records": len(df[df.get('is_combined_result', pd.Series([False]*len(df))) == True]) if not df.empty else 0
    }
    
    return summary


def main():
    """Demonstration of report card data creation functionality."""
    console = Console()
    
    console.print("[bold green]📊 Report Card Data Creation Module for Dane County Schools[/bold green]\n")
    
    # Check if API key is available
    if not os.environ.get('LABKEY_API'):
        console.print("[yellow]⚠ LABKEY_API environment variable not set[/yellow]")
        console.print("Set your LabKey API key to test data retrieval:")
        console.print("export LABKEY_API='your_api_key_here'")
        return 1
    
    try:
        # Test 1: Get raw school data
        console.print("[blue]Test 1: Retrieving raw school virus data...[/blue]")
        
        # Get raw data for testing
        raw_data = retrieve_school_virus_data(max_rows=1000, show_progress=True, test_filter=True)
        
        if raw_data.empty:
            console.print("[red]No raw data available for testing[/red]")
            return 1
        
        console.print(f"✓ Retrieved {len(raw_data)} raw records")
        
        # Test 2: Create report card data for Oregon High School
        console.print(f"\n[blue]Test 2: Creating report card data for Oregon High School...[/blue]")
        
        report_data = create_report_card_data(
            raw_data, 
            test_school="Oregon High School",
            show_progress=True
        )
        
        if not report_data.empty:
            console.print(f"✓ Created {len(report_data)} report card records")
            
            # Show sample data
            console.print("\n[cyan]Sample report card data (first 5 rows):[/cyan]")
            sample_table = Table()
            for col in report_data.columns:
                sample_table.add_column(col, style="white")
            
            for _, row in report_data.head(5).iterrows():
                sample_table.add_row(*[str(val) for val in row])
            
            console.print(sample_table)
            
            # Test 3: Generate Excel output
            console.print(f"\n[blue]Test 3: Generating Excel output...[/blue]")
            
            output_path = Path(__file__).parent / "oregon_high_school_report_card.xlsx"
            save_to_excel(report_data, output_path)
            
            console.print(f"✓ Excel file saved: {output_path}")
            
        else:
            console.print("[yellow]No report card data created for Oregon High School[/yellow]")
        
        # Test 4: Summary statistics
        if not report_data.empty:
            console.print(f"\n[blue]Test 4: Report card data summary...[/blue]")
            summary = get_report_card_summary(report_data)
            
            summary_table = Table(title="Report Card Data Summary")
            summary_table.add_column("Property", style="cyan")
            summary_table.add_column("Value", style="white")
            
            for key, value in summary.items():
                if key not in ["schools", "targets", "assay_systems"]:
                    summary_table.add_row(str(key), str(value))
            
            console.print(summary_table)
        
        console.print("\n[green]✅ All tests completed successfully![/green]")
        console.print("\n[cyan]Primary usage as a module:[/cyan]")
        console.print("from create_report_card_data import create_report_card_data, save_to_excel")
        console.print("from retrieve_virus_data import retrieve_school_virus_data")
        console.print("")
        console.print("raw_df = retrieve_school_virus_data()")
        console.print("report_df = create_report_card_data(raw_df)")
        console.print("save_to_excel(report_df, Path('output.xlsx'))")
        
        return 0
        
    except Exception as e:
        console.print(f"[red]❌ Error: {e}[/red]")
        import traceback
        traceback.print_exc()
        return 1


if __name__ == "__main__":
    exit_code = main()
    sys.exit(exit_code)